PROGRAM MAP_STEEL_MUCG46
! -------------------------------------------------------------------
! Copyright Dr. H. K. D. H. Bhadeshia, University of Cambridge
! MUCG46.FOR
! Department of Materials Science and Metallurgy, 
!   Pembroke St. Cambridge CB2 3QZ
! Version 1.1, May 1990
! Version 1.2, Feb 1991, better Ms calculation
! Version 1.3, May 1991, graphics
! Version 1.4 Consistent with MUCG64
! Version 1.5 New Widmanstatten ferrite logi! WSTION WSTINE
! Version 1.6 REED, REEDI, LOGO, BOUND included
! Version 1.7 July 1993 CG modified
!
! Conversion to Fortran 90 by S.Cardie December 1997
!
! FTVSCLR PROGRAM=.TREAL ERMO:MUCG45 DATA=.DATA NAG
!
! 6 MARCH 1987
!
! TYPICAL DATASET FOLLOWS
!  3 0 (number of steels. zero if rate constants etc not needed)
!  0.39  2.05  0.00  4.08  0.00  0.00  0.00    1
!  0.22  2.03  3.00  0.00  0.00  0.00  0.00    4
!  0.44  1.74  0.67  1.85  0.39  0.83  0.09    3
!   !     SI    MN    NI    MO    CR    V    NUMBER
!
! MAP subroutines used :-     MAP Functions used :-
!
! STEEL :-                    STEEL :-
!
!   MAP_STEEL_AXTO               MAP_STEEL_AFEG
!   MAP_STEEL_GMAAX              MAP_STEEL_CG
!   MAP_STEEL_OMEGA              MAP_STEEL_DAFEG
!   MAP_STEEL_TTTT               MAP_STEEL_DCG
!   MAP_STEEL_WSTINE             MAP_STEEL_ENERGY
!                                MAP_STEEL_FTO1
! UTIL :-                        MAP_STEEL_WSFUN
!
!   MAP_UTIL_ANALY
!   MAP_UTIL_REEDI
!   MAP_UTIL_REED
!
! PROGRAM OUTPUT :
!
! FPRO is the driving force for allotriomorphic ferrite formation by
!     a paraequilibrium mechanism. All driving forces are in units of J/mol and 
!     all calculations are for the paraequilibrium state.
! GMAX is the driving force for nucleation (Fig. 6.2b, Ref. 2).
! CTEMP is the temperature in centigrade.
! XNUCLEUS is the optimum nucleus carbon concentration in mole 
!     fraction.
! XEQ is the paraequilibrium carbon concentration of austenite in 
!     mole fraction.
! XEQ50 is the same as XEQ but allowing for 50 J/mol of 
!     strain energy in the ferrite in order to do calculations for 
!     Widmanstatten ferrite.
! FTO is the driving force for diffusionless transformation (Fig. 
!     6.2c, Ref. 2).
! XTO is the T-zero carbon concentration in mole fraction (Fig. 1.4,
!     Ref. 2).
! XTO400 is the same as XTO, but with 400 J/mol of stored 
!     energy in the ferrite, to allow calculations on bainite (Fig. 1.4, Ref. 2).
! SHEART and DIFFT are the times in seconds of the lower 
!     and upper C-curves of the TTT diagram (Ref. 1).
!
! -------------------------------------------------------------------
!
! Start of Program
!
! -------------------------------------------------------------------

USE MAP_Util_Global_Data
USE MAP_Steel_Free_Energy
USE MAP_Steel_Acquire_Data

IMPLICIT NONE

REAL(KIND=real_8_30), DIMENSION(8) :: C

REAL(KIND=real_8_30), DIMENSION(100) :: BDIF(40), BSH(40), BTEM(40), &
& DFPRO, DDFTO, DT4, DXQ(40)

REAL(KIND=real_8_30) :: &
  CONST, CORR, CTEMP, DIFFT, F, FPRO, FPROA, FSON, FTO, FTOX,&
  GMAX, SHEART, SLOPE, STORE, T, T10, &
  T20, W, X, X1, X44, XBAR, &
  XEQ, XEQ2, XTO, XTO400

REAL(KIND=real_8_30) :: MAP_STEEL_ENERGY, SET_FTO

INTEGER :: COUNTER,COUNTER_MAX,I,IK,IMAX,J2,J5,J6,J98,J99

! -----------------------------------------------------------------------
!											
! Set parameters
!
! -----------------------------------------------------------------------

REAL(KIND=real_8_30),PARAMETER :: &
         W1 = 48570.0E+00,  XA = 0.001E+00,   &
         H  = 38575.0E+00,  S  = 13.48E+00,   &
         STRAIN1 = 0.0E+00, STRAIN2 = 5.0E+01
 
INTEGER, PARAMETER :: START = 473, FINISH = 1173, STEPS = 20, TIMES=1

! -----------------------------------------------------------------------
!
! START, STOP and STEPS define the parameters for loop 27.
!
! -----------------------------------------------------------------------

INTERFACE reed
  MODULE PROCEDURE MAP_UTIL_REED,MAP_UTIL_REEDI
END INTERFACE 

OPEN(UNIT=11,FILE='OUT.DAT')
OPEN(UNIT=12,FILE='PLOT.DAT')
OPEN(UNIT=14,FILE='IN.DAT')
! -----------------------------------------------------------------------
!
! ******* Start of loop '30' *******
!    TIMES controls number of program 'runs'; set as a parameter.
!
! -----------------------------------------------------------------------

  DO 30 I=1,TIMES,1

! -----------------------------------------------------------------------
!
! Initialise variables to zero
!
! -----------------------------------------------------------------------
  IMAX = 0
  X    = 0.0E+00
  DO IK=1,40
    BSH(IK)   = 0
    BDIF(IK)  = 0
    BTEM(IK)  = 0
    DXQ(IK)   = 0
    DT4(IK)   = 0
    DDFTO(IK) = 0
    DFPRO(IK) = 0
  END DO
! -----------------------------------------------------------------------
!
! Get input variables from user; check values using REED and BOUND
!
! -----------------------------------------------------------------------
  CALL LOGO()

  WRITE(6,1085)
  CALL reed(J6)

  CALL INPUT_TYPE(C)
 !
 ! Write header info to 'out.dat'
 !
  WRITE(11,1040)
  WRITE(11,*)
  WRITE(11,1045) J6
  WRITE(11,1025) (C(J5),J5 = 1,7)
!
! Write header info to 'plot.dat'
!
  WRITE(12,1300)
  !      IMAX       = IMAX+1
  !      BSH(IMAX)  = SHEART
  !      BDIF(IMAX) = DIFFT
  !      BTEM(IMAX) = CTEMP
! -----------------------------------------------------------------------
!
! END OF DATA ACQUISITION SECTION
!
! -----------------------------------------------------------------------
!
! Call OMEGA to get values of W, XBAR, T10 and T20. OMEGA calculates 
!  the carbon-carbon interaction energy in austenite, as a function 
!  of alloy composition,
!
! -----------------------------------------------------------------------

  CALL MAP_STEEL_OMEGA(C,W,XBAR,T10,T20,0)

  WRITE(11,1025) (C(J5),J5 = 1,7)

! -----------------------------------------------------------------------
!
! Initialise many variables used in 'loop 27', the main calculation loop
!
! -----------------------------------------------------------------------
       FTO = -1.0
       X1  = C(1)

       WRITE (11,*)
       WRITE (11,1020) X1,T10,T20,W
       WRITE (11,1040)
       WRITE (11,1030)

       XEQ    = 0.2E+00
       XEQ2   = 0.15E+00
       XTO    = 0.07E+00
       XTO400 = 0.06E+00
       X44    = 0.1E+00
       CONST  = 1.0E+00
       CORR   = 1.0E+00
       SLOPE  = 1.0E+00
! -----------------------------------------------------------------------
!
! 'COUNTER' counts how many times we've been round loop 27;
!   initialised to zero here.
! 'COUNTER_MAX' is the maximum number of iterations; 
!   initialised to 1000 here.
!
! -----------------------------------------------------------------------

       COUNTER     = 0
       COUNTER_MAX = 1000

! -----------------------------------------------------------------------
!
! ******* Start of loop '27' *******
!
!  Loop 27 calculates and outputs the values of 16 variables for 
!    temperatures between START and FINISH in increments of STEPS
!
!  The 16 variables are :-
!
!   FPRO,FPROA,GMAX,CTEMP,X,FSON,XEQ,XEQ2,FTO,XTO,J98,X44,XTO400,J99,
!   SHEART and DIFFT
! 
! -----------------------------------------------------------------------

       DO J2=START,FINISH,STEPS

! -----------------------------------------------------------------------
!
! Set loop variables.
!
! -----------------------------------------------------------------------

        COUNTER = COUNTER+1
        XTO     = XTO+0.0001E+00
        XTO400  = XTO400+0.0001E+00
        X44     = 0.3*XEQ

! -----------------------------------------------------------------------
!
! T is temperature in kelvin, set by loop counter.
!
! -----------------------------------------------------------------------

        T       = J2*1.0E+00

! -----------------------------------------------------------------------
!
! CTEMP is temperature in deg. C
!
! -----------------------------------------------------------------------

        CTEMP   = T-2.73E+02

! -----------------------------------------------------------------------
!
! Set F using function ENERGY, passing parameters T, T10 and T20. 
!  ENERGY calculates the free energy change for the transformation of 
!  austenite to ferrite of the same chemical composition.
! F is used by subroutines TEQ_CHECK, FPRO_GMAX and AXTO.
!
! -----------------------------------------------------------------------

        F  = MAP_STEEL_ENERGY(T,T10,T20)

! -----------------------------------------------------------------------
!
! The following section sets XEQ and XEQ2, which are two of our output 
!   variables.  The subroutine TEQ_CHECK is used to set first XEQ using
!   the parameter STRAIN1, then XEQ2 using STRAIN2.
!
! -----------------------------------------------------------------------

        CALL TEQ_CHECK(XEQ,F,T,STRAIN1,W)
        CALL TEQ_CHECK(XEQ2,F,T,STRAIN2,W)

! -----------------------------------------------------------------------
!
! Exit loop 27 prematurely if XEQ value TRUE for check below
!
! -----------------------------------------------------------------------
 
        IF (XEQ .LT. X1+0.0004) EXIT

! -----------------------------------------------------------------------
!
! Set FPRO, FPROA, FSON, GMAX and X using sub. FPRO_GMAX.
!  This sub. calls MAP_STEEL_GMAAX to calculate GMAX and X.
!
! -----------------------------------------------------------------------

        CALL FPRO_GMAX(FSON,FPRO,FPROA,GMAX,&
                       F,T,W,W1,X,XA,XEQ,X1)

! -----------------------------------------------------------------------
!
! X44 was initialised above and is set here using the GET_X44 subroutine.
!
! -----------------------------------------------------------------------

        CALL GET_X44(X44,T,W,XEQ)

! -----------------------------------------------------------------------
!
! Set value for FTO, using function SET_FTO, which calls 
!   MAP_STEEL_FTO1 if required.  FTOX holds the current
!   value of FTO, prior to checking.
!
! -----------------------------------------------------------------------

        FTOX = FTO
        FTO  = SET_FTO(FTOX,H,S,X1,T,W,W1,F)
 
! -----------------------------------------------------------------------
!
! AXTO sets final (for loop pass) values of XTO, XTO400, J98 and J99.
!   AXTO calculates the carbon concentration (mole fraction) at the
!   T0 phase boundary at a specified temperature.
! Values are set for two values of STORE.
!
! -----------------------------------------------------------------------

        STORE = 0.0E+00
        CALL MAP_STEEL_AXTO(H,S,XTO,T,W,W1,F,&
     &       STORE,COUNTER_MAX,J98)

	    STORE = 4.0E+02
        CALL MAP_STEEL_AXTO(H,S,XTO400,T,W,W1,F,&
     &       STORE,COUNTER_MAX,J99)

! -----------------------------------------------------------------------
!
! TTTT sets (for loop pass) values of SHEART and DIFFT.
!   TTTT calculates incubation times due to displacive transformations.
!	Note that X44 may be reset depending on value of SHEART
!
! -----------------------------------------------------------------------

        CALL MAP_STEEL_TTTT(T,GMAX,SHEART,DIFFT)
        IF (X44 .EQ. 0.0) SHEART = 1.0E+20 
    
! -----------------------------------------------------------------------
!
! We now have all sixteen output variables, so we can write results 
!   to output file.
!
! -----------------------------------------------------------------------

   WRITE(11,1005) FPRO,FPROA,GMAX,CTEMP,X,FSON,XEQ,XEQ2,FTO,XTO,&
                  J98,X44,XTO400,J99,SHEART,DIFFT

! -----------------------------------------------------------------------
!
! End of main calculation section.  Next section produces curves based
!  on results calculated above.
!
! -----------------------------------------------------------------------
 
        IMAX       = IMAX+1
        BSH(IMAX)  = SHEART
        BDIF(IMAX) = DIFFT
        BTEM(IMAX) = CTEMP

        WRITE(12,1090)IMAX,BSH(IMAX),BDIF(IMAX),BTEM(IMAX) 

        DXQ(COUNTER)   = XEQ
        DDFTO(COUNTER) = FTO
        DT4(COUNTER)   = CTEMP
        DFPRO(COUNTER) = FPRO
   END  DO
! -----------------------------------------------------------------------
!
! ******* End of loop '27' *******
!
! -----------------------------------------------------------------------

       COUNTER = COUNTER-1

       WRITE(11,*)
       WRITE(11,1120)

! -----------------------------------------------------------------------
!
! REGRESS calls
! MAP_UTIL_ANALY, a linear regression subroutine which determines the 
!  intercept and gradient of the best fit line between two variables, 
!  together with the correlation coefficient.
! X1 is C(1).
!
! -----------------------------------------------------------------------
 
       CALL REGRESS(COUNTER,X1,DT4,DDFTO,DFPRO)

       WRITE(11,1040)

30    CONTINUE
! -----------------------------------------------------------------------
!
! ******* End of loop '30' *******
!
! -----------------------------------------------------------------------
      CLOSE(11)
	  CLOSE(12)
	  CLOSE(14)
      STOP
! -----------------------------------------------------------------------
!
! Format statements grouped here ...
!
! -----------------------------------------------------------------------
1005  FORMAT (1X, F7.0, 2F7.0, F6.0, D10.2, F8.0, &
     &2F9.4, F7.0, F8.4, 1X, I2, 2F8.4, 1X, I2, D11.2, D11.2)
1010  FORMAT(' CTEMP=',F8.0,' ALPHA-1=',D10.4)
1015  FORMAT (7F6.2,I5)
1020  FORMAT (17X,'CARBON CONTENT=',F10.5,5X,'T10=',F10.6,&
     &5X,'T20=',F10.6,5X,'WGAMMA=',F7.0)
1025  FORMAT (6X,'C=',F8.4,6X,'SI=',F8.4,6X,'MN=',F8.4, &
     &6X,'NI=',F8.4,6X,'MO=',F8.4,6X,'CR=',F8.4,6X,'V=',F8.4)
1030  FORMAT('   FPRO  FPROA   GMAX  CTEMP',&
      2X,  'X NUCLEUS',2X,  'FSON',6X,      'XEQ',5X,    'XEQ50',&
      2X,   'FTO',6X,      'XTO',8X,        'X44'  ,		   &
	  5X, 'XTO400' ,6X,      'SHEART'   ,5X,  'DIFFT'   )
1040  FORMAT('--------------------------------------------------',&
     &'-------------------------------',						   &
	 &'----------------------------------------------')
1045  FORMAT (16X, '*******NUMBER'  ,I5,11X,  '*********')
1085  FORMAT(////19X,' Identification Number ?')
1090  FORMAT(I5,3D12.2)
1120  FORMAT('    ***** FTO VERSUS TEMPERATURE ****   ')
1300  FORMAT (11X,'SHEART',7X,'DIFFT',7X,'CTEMP')
      END PROGRAM MAP_STEEL_MUCG46
! -----------------------------------------------------------------------
! 
! Subroutines
!
! -----------------------------------------------------------------------
!
! Subroutine MAP_UTIL_ANALY
! 
! -----------------------------------------------------------------------
    SUBROUTINE MAP_UTIL_ANALY(J8,J9,CONST,SLOPE,CORR,X,Y)
      USE MAP_Util_Global_Data
      IMPLICIT NONE
!
!	  Arguments
!
      REAL(KIND=real_8_30),INTENT(IN),DIMENSION(100) :: X,Y
      REAL(KIND=real_8_30),INTENT(OUT)  :: CONST,SLOPE,CORR
      INTEGER,INTENT(IN) :: J8,J9
!
!     Local Variables
!
      INTEGER              :: I,I1
      REAL(KIND=real_8_30) :: AX =0.0E+00,AY =0.0E+00, &
                              AX2=0.0E+00,AY2=0.0E+00, &
                              AXY=0.0E+00

        I1=J8-J9

        DO I=1,I1
         AX  = AX+X(I)
         AY  = AY+Y(I)
         AXY = AXY+X(I)*Y(I)
         AX2 = AX2+X(I)*X(I)
         AY2 = AY2+Y(I)*Y(I)
        END DO

        CONST =(AY*AX2-AX*AXY)/(I1*AX2-AX*AX)
        SLOPE =((I1*AXY-AX*AY)/(I1*AX2-AX*AX))
        CORR  =(I1*AXY-AX*AY)/(SQRT((I1*AX2-AX*AX)* &
               (I1*AY2-AY*AY)))
     
      RETURN
    END SUBROUTINE MAP_UTIL_ANALY
! -----------------------------------------------------------------------
!
! Subroutine MAP_STEEL_AXTO
!
! To calculate the carbon concentration (mole fraction) at the
!  T-zero phase boundary at a specified temperature.
!		  
! H K D H Bhadeshia, University of Cambridge
!
! -----------------------------------------------------------------------
    SUBROUTINE MAP_STEEL_AXTO(H,S,XTO,T,W,W1,F,STORE,JN,J)
      USE MAP_Util_Global_Data
      USE MAP_Steel_Free_Energy

      IMPLICIT NONE
!
!  Arguments
!
      REAL(KIND=real_8_30),INTENT(IN) ::  H,S,T,W,W1,F,STORE
      INTEGER,INTENT(IN) :: JN

	  REAL(KIND=real_8_30),INTENT(OUT) :: XTO
	  INTEGER,INTENT(OUT)              :: J

!
! Local Variables
!
      REAL(KIND=real_8_30)          :: DA,DFTO,G9
!
! XTO is the carbon concentration at the T-zero boundary
! T is the absolute temperature
! STORE is the stored energy in Joules per mole
! H, S, H1, S1, W, W1, F, AJ, AJ1 are thermodynamic quantities
!  defined in subroutines OMEGA, ENERGY and main program
!
! Initialize J
!
        J=0
!
! Calculate the driving force for the transformation of
!  austenite to ferrite without any change in chemical composition,
!  allowing for Zener ordering
!

        DO
          DFTO=MAP_STEEL_FTO1(F,H,S,T,W,W1,XTO)+STORE
          DA=ABS(DFTO)
!
! Solve for XTO using Newton's iterative method
!
          J=J+1
          IF (DA .LE. 1.0E+01) EXIT
!
! Obtain differential of FTO1 with respect to XTO
!

          G9=MAP_STEEL_G91(F,H,S,T,W,W1,XTO)
!
! Modify original guess of XTO, assuming that convergence occurs
!  within nine iterations
!
          IF (J .GE. JN) EXIT

          XTO=XTO-DFTO/G9
          IF (XTO .LE. 1.0E-12) THEN
		     XTO=0.0000E+00
             EXIT
          END  IF

        END DO

        RETURN
      END SUBROUTINE MAP_STEEL_AXTO
! -----------------------------------------------------------------------
!
! Subroutine MAP_STEEL_GMAAX
!
! -----------------------------------------------------------------------
  SUBROUTINE MAP_STEEL_GMAAX(A1,A,W1,F,T,X,AFE)
    USE MAP_Util_Global_Data
    IMPLICIT NONE
!
! Arguments
!          
    REAL(KIND=real_8_30),INTENT(OUT) :: A1,X
    REAL(KIND=real_8_30),INTENT(IN)  :: A,W1,F,T,AFE
!
! Local Variables
!
    REAL(KIND=real_8_30) :: A1FE,D1,B1,B2,B3,DA1,DA1FE,DA2,&
&                           ERROR,TEST
!	
! AVER=1.2
! CALCULATION OF THE OPTIMUM NUCLEUS ! CONTENT AND ACTIVITY OF ! IN
! FERRITE NUCLEUS
!
! Estimate a value for X, the optimum carbon concentration of
!  the ferrite nucleus, in mole fraction
! February 1991

    X=6.3998E-07*T-3.027E-04
    IF(X .LE. 0.0E+00)  X=1.0E-13
    IF(X .GT. 0.32E-03) X=0.32E-03
!
! Estimation complete
!
    DO
      D1=SQRT(9.0E+00-6.0E+00*X*(2.0E+00*AJ(T,W1)+3.0E+00) &
            +(9.0E+00+16.0E+00*AJ(T,W1))*X**2)

      B1=((D1-3.0E+00+5.0E+00*X)/(D1+3.0E+00-5.0E+00*X))
      B2=((3.0E+00-4.0E+00*X)/X)
      B3=(H_T(T) - S_T(T)*T + 4.0E+00*W1)/(R*T)

      A1=LOG(B1**4*B2**3)+B3

      IF(X .GT. 1.0E-08)THEN
        A1FE=LOG(1.0E+00-X)
      ELSE
        A1FE=X
      ENDIF

      TEST=F+R*T*(A1FE-AFE) - R*T*(A1-A)
!
! Newton iteration
!
      IF (ABS(TEST) .GT. 10.0) THEN
        DA1=(3.0E+00*X/(3.0E+00-4.0D+00*X))*( &
     &  (4.0E+00*X-3.0E+00)/(X**2)-4.0E+00/X)
        DA2=(0.5E+00/D1)*(-12.0E+00*AJ(T,W1)-18.0E+00+  &
     &  18.0E+00*X+32.0E+00*AJ(T,W1)*X)
        DA2=4.0E+00*(((DA2+5.0E+00)/(D1-3.0E+00+5.0E+00*X)) &
     &  -((DA2-5.0E+00)/(D1+3.0E+00-5.0E+00*X)))
        DA1=DA1+DA2
        DA1FE=1.0E+00/(X-1.0E+00)
        ERROR=TEST/(R*T*(DA1FE-DA1))
        IF (ERROR .GT. X) ERROR = 0.3E+00*X
        X=ABS(X-ERROR)
      ELSE
	     EXIT
      ENDIF

    END DO

    RETURN
  END SUBROUTINE MAP_STEEL_GMAAX
! -----------------------------------------------------------------------
!
! Subroutine MAP_STEEL_OMEGA
!   Calculates the carbon-carbon interaction energy in austenite as 
!   a function of alloy composition.
!   Result is in J/m  **7 OCTOBER 1981**
!
! -----------------------------------------------------------------------
    SUBROUTINE MAP_STEEL_OMEGA(C,W,XBAR,T10,T20,J)
      USE MAP_Util_Global_Data
      IMPLICIT NONE
!
! Arguments
!
      REAL(KIND=real_8_30),INTENT(INOUT),DIMENSION(8) :: C
      INTEGER,INTENT(IN) :: J
      REAL(KIND=real_8_30),INTENT(OUT) :: W,XBAR,T10,T20
!
! Local Variables
!
      REAL(KIND=real_8_30),DIMENSION(8) :: P,Y
      REAL(KIND=real_8_30) :: B1,B2=0.0E+00,B3=0.0E+00
      INTEGER :: IU

      IF(J .EQ. 1) THEN
         B1=C(1)+C(2)+C(3)+C(4)+C(5)+C(6)+C(7)+C(8)
      ELSE
         C(8)=C(1)+C(2)+C(3)+C(4)+C(5)+C(6)+C(7)
         C(8)=100.0E+00-C(8)
         C(8)=C(8)/55.84E+00
         C(1)=C(1)/12.0115E+00
         C(2)=C(2)/28.09E+00
         C(3)=C(3)/54.94E+00
         C(4)=C(4)/58.71E+00
         C(5)=C(5)/95.94E+00
         C(6)=C(6)/52.0E+00
         C(7)=C(7)/50.94E+00
         B1  =C(1)+C(2)+C(3)+C(4)+C(5)+C(6)+C(7)+C(8)
      END  IF  

      DO 107 IU=2,7
        Y(IU)=C(IU)/C(8)
107   CONTINUE

      DO 106 IU=1,8
        C(IU)=C(IU)/B1
106   CONTINUE

      T10=   Y(2)*(-3)+Y(3)*2+Y(4)*12+Y(5)*(-9)+Y(6)*(-1)+Y(7)*(-12)
      T20=-3*Y(2)-37.5*Y(3)-6*Y(4)-26*Y(5)  -19*Y(6)  -44*Y(7)

      XBAR=C(1)
      XBAR=INT(10000.0E+00*XBAR)
      XBAR=XBAR/10000

!
!     Polynomials representing carbon-carbon interaction energy (J/mol)
!     in austenite as a function of the molecular fraction of individual solutes.
!
      P(2)=-2.4233E+07+6.9547E+07*C(2)
      P(2)=+3.864E+06+P(2)*C(2)
      P(2)=+45802.87+(-280061.63+P(2)*C(2))*C(2) 
      P(2)=2013.0341+(763.8167+P(2)*C(2))*C(2)
      P(3)=2.0119E+06+(3.1716E+07-1.3885E+08*C(3))*C(3)
      P(3)=+6287.52+(-21647.96+P(3)*C(3))*C(3)
      P(3)=2012.067+(-1764.095+P(3)*C(3))*C(3)
      P(4)=-2.4968E+07+(1.8838E+08-5.5531E+08*C(4))*C(4)
      P(4)=-54915.32+(1.6216E+06+P(4)*C(4))*C(4)
      P(4)=2006.8017+(2330.2424+P(4)*C(4))*C(4)
      P(5)=-1.3306E+07+(8.411E+07-2.0826E+08*C(5))*C(5)
      P(5)=-37906.61+(1.0328E+06+P(5)*C(5))*C(5)
      P(5)=2006.834+(-2997.314+P(5)*C(5))*C(5)
      P(6)=+8.5676E+06+(-6.7482E+07+2.0837E+08*C(6))*C(6)
      P(6)=+33657.8+(-566827.83+P(6)*C(6))*C(6)
      P(6)=2012.367+(-9224.2655+P(6)*C(6))*C(6)
      P(7)=+5411.7566 &
     &+(250118.1085-4.1676E+06*C(7))*C(7)
      P(7)=2011.9996+(-6247.9118+P(7)*C(7))*C(7)

      DO 108 IU=2,7
        B3=B3+P(IU)*Y(IU)
        B2=B2+Y(IU)
108   CONTINUE

      IF (B2 .EQ. 0.0E+00) THEN
	     W=8.054E+03
      ELSE
         W=(B3/B2)*4.187E+00
      END  IF

      RETURN
    END SUBROUTINE MAP_STEEL_OMEGA
! -----------------------------------------------------------------------
!
! Subroutine MAP_STEEL_TTTT
!
! -----------------------------------------------------------------------
  SUBROUTINE MAP_STEEL_TTTT(T,GMAX,SHEART,DIFFT)
    USE MAP_Util_Global_Data
    IMPLICIT NONE
!
! Arguments
!
    REAL(KIND=real_8_30),INTENT(IN)  :: GMAX,T
    REAL(KIND=real_8_30),INTENT(OUT) :: DIFFT,SHEART


      SHEART=EXP((0.2432E+06/(R*T))-0.135E+03+20.0*LOG(T)	&
      -5*LOG(ABS(GMAX)))
      DIFFT=EXP((0.6031E+06/(R*T))-0.1905E+03+20.0*LOG(T)	&
      -4.0E+00*LOG(ABS(GMAX)))
 
    RETURN
  END SUBROUTINE MAP_STEEL_TTTT
! -----------------------------------------------------------------------
!
! Subroutine MAP_STEEL_WSTINE
!
! -----------------------------------------------------------------------
      SUBROUTINE MAP_STEEL_WSTINE(DFPRO,WS,WS1,J8)
! Version 1.1 May 1991
! August 1995
      USE MAP_Util_Global_Data
      IMPLICIT NONE
!
! Arguments
!
      REAL(KIND=real_8_30),INTENT(IN),DIMENSION(40) :: DFPRO
      INTEGER,INTENT(IN) :: J8

      REAL(KIND=real_8_30),INTENT(OUT) :: WS,WS1
!
! Local Variables
!
      REAL(KIND=real_8_30),EXTERNAL :: MAP_STEEL_WSFUN
      REAL(KIND=real_8_30) :: KTEMP,V14

      INTEGER :: I,J=0

      DO I=1,J8
        KTEMP=200.00E+00+T0+(I-1)*20.0E+00
        V14=MAP_STEEL_WSFUN(KTEMP)
!
! Stored energy for growth of Wid ferrite = 50 J/mol
        IF(DFPRO(I) .GT. -50.0E+00) EXIT
!
! WS1 is the lower intersection of Gn and Fpro curves
        IF(DFPRO(I) .LE. V14 .AND. J .EQ. 0) WS1=KTEMP-T0
        IF(DFPRO(I) .GE. V14) J=1
!
! WS is the upper intersection of Gn and Fpro curves
!
        IF(DFPRO(I) .LE. V14) WS=KTEMP-T0
      END DO
 
      RETURN
      END SUBROUTINE MAP_STEEL_WSTINE
! -----------------------------------------------------------------------
!
! Subroutine TEQ_CHECK
!
! -----------------------------------------------------------------------
     SUBROUTINE TEQ_CHECK(XEQ,F,T,STR,W)
       USE MAP_Util_Global_Data
	   USE MAP_Steel_Activity
       IMPLICIT NONE
!
! Arguments
!
        REAL(KIND=real_8_30),INTENT(INOUT) :: XEQ
        REAL(KIND=real_8_30),INTENT(IN)    :: F,STR,T,W
!
! Local variables
!
        REAL(KIND=real_8_30) :: ETEQ,TEQ

        DO
          TEQ = R*T*MAP_STEEL_AFEG(T,W,XEQ)-F-STR

          IF (ABS(TEQ) .GE. 1.0)  THEN
             ETEQ = MAP_STEEL_DAFEG(T,W,XEQ)*R*T
             XEQ  = XEQ-TEQ/ETEQ
		  ELSE
		     EXIT
          END IF
        END DO

       RETURN
     END SUBROUTINE TEQ_CHECK
! -----------------------------------------------------------------------
!
!  SUBROUTINE GET_X44
!
! -----------------------------------------------------------------------
   SUBROUTINE GET_X44(X,T,W,XEQ)
     USE MAP_Util_Global_Data
     USE MAP_Steel_Activity
     IMPLICIT NONE
! 
! Arguments
!
       REAL(KIND=real_8_30),INTENT(IN)  :: T,W,XEQ
       REAL(KIND=real_8_30),INTENT(OUT) :: X
!
! Local Variables
!
       REAL(KIND=real_8_30),EXTERNAL :: MAP_STEEL_WSFUN
       REAL(KIND=real_8_30) :: A,AEQ,AFE,AFEQ,DA,DAFE,DF,F,V14

! -----------------------------------------------------------------------
!
! V14 is the free energy needed to nucleate 
!    Widmanstatten ferrite. Calculated by WSFUN, using  
!    variable T and parameter T0.
! -----------------------------------------------------------------------

       V14 = MAP_STEEL_WSFUN(T)
 
       IF (V14 .GE. 0.0E+00)  THEN
          X=99999999999.99999
       ELSE

         AEQ  = MAP_STEEL_CG(T,W,XEQ)
         AFEQ = MAP_STEEL_AFEG(T,W,XEQ)

         DO
             A     = MAP_STEEL_CG(T,W,X)
             AFE   = MAP_STEEL_AFEG(T,W,X)
             F     = R*T*(X*(AEQ-A)+(1-X)*(AFEQ-AFE))
             F     = F-V14

             IF (ABS(F) .GE. 10.0) THEN
                DA   = MAP_STEEL_DCG(T,W,X)
                DAFE = MAP_STEEL_DAFEG(T,W,X)
                DF   = R*T*(AEQ-X*DA-AFEQ+X*DAFE-DAFE-A+AFE)
                X    = X-F/DF
             ELSE
                EXIT
             END  IF       
          END  DO

        END  IF

      RETURN
    END SUBROUTINE GET_X44
! -------------------------------------------------------------------
!
! Functions
!
! -----------------------------------------------------------------------
!
! Function MAP_STEEL_ENERGY
! 
! -----------------------------------------------------------------------
    FUNCTION MAP_STEEL_ENERGY(T,T10,T20)
      USE MAP_Util_Global_Data
      IMPLICIT NONE
!
! Arguments
!
      REAL(KIND=real_8_30),INTENT(IN) :: T,T10,T20
      REAL(KIND=real_8_30) :: MAP_STEEL_ENERGY
!
! Local Variables
!
      REAL(KIND=real_8_30) :: F
      REAL(KIND=real_8_30) :: T7,T8

        T7=T -100.0E+0*T20
        T8=T7-1140.0E+0

        IF (T7.LT.300) F=1.38*T7-1499.0
        IF (T7.LT.700.AND.T7.GE.300)  F=1.65786*T7-1581.0
        IF (T7.LT.940.AND.T7.GE.700)  F=1.30089*T7-1331.0

        IF (T7.GE.940) THEN
           F=T8*(-3.58434E-9*T8 + 2.70013E-6) - 1.04923E-3
           F=T8*(F*T8 + 0.26557E+0) - 8.88909E+0
        END  IF
 
      MAP_STEEL_ENERGY=(1.41E+02*T10 + F)*4.187E+00

      RETURN
    END FUNCTION MAP_STEEL_ENERGY
!-----------------------------------------------------------------------
!
! Function MAP_STEEL_WSFUN
!
!-----------------------------------------------------------------------
    REAL(KIND=real_8_30) FUNCTION MAP_STEEL_WSFUN(KTEMP)
      USE MAP_Util_Global_Data
      IMPLICIT NONE
      REAL(KIND=real_8_30),INTENT(IN) :: KTEMP

! Version 1.1 May 1991
!
! Gn curve for Widmanstatten ferrite nucleation, Acta Metall. 1981
! J/mol
! based on FPRO rather than GMAAX
!
      MAP_STEEL_WSFUN=3.251745029E+00*(KTEMP-T0)-2183.014E+0
!
! When equal to or greater than zero, no problem with nucleation
!
      RETURN
    END FUNCTION MAP_STEEL_WSFUN
! -----------------------------------------------------------------------
!
! Function SET_MS
!
! -----------------------------------------------------------------------
       REAL(KIND=real_8_30) FUNCTION SET_MS(X,CONST,SLOPE)
       USE MAP_Util_Global_Data
       IMPLICIT NONE
!
!Arguments
!
       REAL(KIND=real_8_30), INTENT(IN) :: X,CONST,SLOPE

       SET_MS = (-1086.9E+00 + 2941.8E+00*X		&
     &                       - 9.6016E+05*X**2	&
     &                       + 2.5382E+07*X**3	&
     &                       - 1.8488E+08*X**4	&
     &                       - CONST)/SLOPE

       RETURN
       END FUNCTION SET_MS
! -----------------------------------------------------------------------
!
! Function SET_FTO
!
! -----------------------------------------------------------------------
    FUNCTION SET_FTO(FTO,H,S,X1,T,W,W1,F)
      USE MAP_Util_Global_Data
      USE MAP_Steel_Free_Energy
      IMPLICIT NONE
!
! Arguments
!
      REAL(KIND=real_8_30)             :: SET_FTO
      REAL(KIND=real_8_30), INTENT(IN) :: FTO,H,S,X1,T,W,W1,F

        IF (FTO .GE. 0.0E+00) THEN
            SET_FTO = 0.0E+00
        ELSE
            SET_FTO = MAP_STEEL_FTO1(F,H,S,T,W,W1,X1)
        END IF

    END FUNCTION SET_FTO
! -----------------------------------------------------------------------
!
! Subroutine REGRESS
!   Void context: no return
!
! -----------------------------------------------------------------------

  SUBROUTINE REGRESS(COUNTER,X1,DT4,DDFTO,DFPRO)
  USE MAP_Util_Global_Data
  IMPLICIT NONE
!
! Arguments
!
    REAL(KIND=real_8_30),INTENT(IN),DIMENSION(100) :: DFPRO, DDFTO, DT4
    REAL(KIND=real_8_30),INTENT(IN) :: X1

    INTEGER,INTENT(IN) :: COUNTER
!
! Local Variables
!
    INTEGER :: J9=10
    REAL(KIND=real_8_30) :: BS,CONST,CORR,MS,SLOPE,WS,WS1,XMS
    REAL(KIND=real_8_30) :: SET_MS

!  -----------------------------------------------------------------------
!
! MAP_UTIL_ANALY is a linear regression subroutine which determines the 
!  intercept and gradient of the best fit line between two variables, 
!  together with the correlation coefficient.
! X1 is C(1).
!
! -----------------------------------------------------------------------
       CALL MAP_UTIL_ANALY(COUNTER,J9,CONST,SLOPE,CORR,DT4,DDFTO)

       WRITE(11,*)
       BS = (-400.0-CONST)/SLOPE
! -----------------------------------------------------------------------
!
! WSTINE calculates the upper and lower intersections of the Gn and 
!  Fpro curves, returned as WS and WS1 respectively.
!
! -----------------------------------------------------------------------
       CALL MAP_STEEL_WSTINE(DFPRO,WS,WS1,COUNTER)
! -----------------------------------------------------------------------
!
! Modification made February 1991 to allow for large carbon concentrations
!  Based on Metal Science paper on thermodynamics of martensitic 
!  transformations in plain carbon steels
!
! -----------------------------------------------------------------------
       XMS = X1
       IF (XMS .GT. 0.0594E+00)  XMS = 0.0594E+00
       MS = SET_MS(XMS,CONST,SLOPE)
      
       WRITE(11,1100) WS1,WS

       IF (WS .LT. BS) THEN
          BS = WS
          WRITE(11,1115) BS
       ELSE
          WRITE(11,1117) BS
       ENDIF

       WRITE(11,1105) MS

    RETURN

1100   FORMAT(' WIDMANSTATTEN START TEMPERATURES ',F7.0,' & ',F7.0)
1105   FORMAT('     MARTENSITE START TEMPERATURE=',F7.0,' C')
1115   FORMAT(' NUCLEATION LIMITED BAINITE START TEMP=',F7.0,' C')
1117   FORMAT(' GROWTH LIMITED BAINITE START TEMP=',F7.0,' C')


END SUBROUTINE REGRESS
! -----------------------------------------------------------------------
!
! Subroutine FPRO_GMAX
!  Calculates FPRO,FPROA,FSON,GMAX and X
!
! -----------------------------------------------------------------------
  SUBROUTINE FPRO_GMAX(FSON,FPRO,FPROA,GMAX,&
                       F,T,W,W1,X,XA,XEQ,X1)
    USE MAP_Util_Global_Data
    USE MAP_Steel_Activity
    IMPLICIT NONE
!
! Arguments
!
    REAL(KIND=real_8_30),INTENT(INOUT) :: FSON,FPRO,FPROA,GMAX,X
                                          
    REAL(KIND=real_8_30),INTENT(IN) :: F,T,W,W1,&
                                       XA,XEQ,X1
!
! Local Variables
!
    REAL(KIND=real_8_30) :: A,A1,AEQ,AFE,AFEQ

        AEQ  = MAP_STEEL_CG(T,W,XEQ)
        AFEQ = MAP_STEEL_AFEG(T,W,XEQ)

        A    = MAP_STEEL_CG(T,W,X1)
        AFE  = MAP_STEEL_AFEG(T,W,X1)

        FSON  = R*T*(XA*(AEQ-A)+(1-XA)*(AFEQ-AFE))
        FPRO  = R*T*(X1*(AEQ-A)+(1-X1)*(AFEQ-AFE))
        FPROA = FPRO*((XEQ-XA)/(XEQ-X1))

        CALL MAP_STEEL_GMAAX(A1,A,W1,F,T,X,AFE)
        GMAX = R*T*(A1-A)

    RETURN
  END SUBROUTINE FPRO_GMAX
! -----------------------------------------------------------------------



